(* Julia Set in Caml Light for DOS *)
(* Original program by AdrienC21; https://github.com/AdrienC21/fractals-caml *)
(* Modifications for Caml Light for DOS 0.7 by allu, September 2024 *)

#open "graphics";;
open_graph "";;
clear_graph ();;

let diverge_julia a b cr ci n =
let x = ref a in
let y = ref b in
let xtemp=ref 0. in
let ytemp=ref 0. in
let k = ref 0 in
while ((!x *. !x+. !y *. !y) < 4.)&&(!k<n) do
xtemp:=(!x)*. (!x)-. (!y)*. (!y)+. cr;
ytemp:=2. *. (!x) *.(!y)+. ci;
x:= !xtemp;
y:= !ytemp;
k:= !k+1;
done;
(!k);;

let julia cr ci n d =
clear_graph ();
let topx=size_x() and topy=size_y() and k = ref 0 in
for x=(-topx/2) to (topx/2) do
for y=(-topy/2) to (topy/2) do
k:=((diverge_julia ((float_of_int x) /. d) ((float_of_int y) /. d) cr ci n)*10);
set_color (rgb (255-(!k)/2) (255-(!k)/2) (255-(!k)));
plot (x+(topx/2)) (y+(topy/2));
done;
done;;

try
  julia (-0.3380) (-0.6230) 150 300.;
  while true do
    (* Loop until key pressed: 1 - 5: draw Julia Set; ESC or q: terminate program *)
    let e = wait_next_event [Key_pressed] in
      if e.keypressed then begin
        match e.key with
         `q` | `Q` | `\027` -> raise Exit
         |`1` -> julia (-0.181) (-0.667) 150 400.
         |`2` -> julia (0.285) (0.01) 300 300.
         |`3` -> julia (-0.7927) (0.1609) 500 300.
         |`4` -> julia (0.32) (0.043) 150 300.
         |`5` -> julia (-0.3380) (-0.6230) 150 300.
      end
  done
  with Exit ->
    close_graph()
;;
exit 0;;
